using System;
using System.Collections.Generic;
using System.IO;

using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.Component.Interop;
using System.Text;

namespace Macaw.BizTalk.Pipeline
{
    [ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
    [ComponentCategory(CategoryTypes.CATID_Any)]
    [System.Runtime.InteropServices.Guid("bab90b6a-9493-4807-a1db-ad87499a7057")]
    public class BuildSourceFileName : PipelineComponentWithResourceBase
    {
        public BuildSourceFileName()
            : base("BuildSourceFileName", Properties.Resources.ResourceManager)
        { }

        private List<LiteralOrProperty> _nameParts =
            new List<LiteralOrProperty>();
        public List<LiteralOrProperty> NameParts
        {
            get { return _nameParts; }
        }

        protected override bool IsEnabled()
        {
            return (base.IsEnabled() && (NameParts.Count > 0));
        }

        protected override IBaseMessage ExecuteInternal(IPipelineContext pContext, IBaseMessage pInMsg)
        {
            StringBuilder fileName = new StringBuilder();

            foreach (LiteralOrProperty litProp in NameParts)
            {
                fileName.Append(litProp.GetValue(pInMsg.Context));
            }

            FileAdapterMessageContext fileCtx = new FileAdapterMessageContext(pInMsg.Context);
            // set file name
            fileCtx.SourceFileName = fileName.ToString();

            return pInMsg;
        }

        #region IPersistPropertyBag implementation

        public override void Load(IPropertyBag propertyBag, int errorLog)
        {
            base.Load(propertyBag, errorLog);

            int count = base.ReadProperty<int>(
                propertyBag, BuildSourceFileNameBagNames.Count, 0);

            for (int index = 0; index < count; index++)
            {
                LiteralOrProperty ctxProp = new LiteralOrProperty();

                ctxProp.Key = ReadProperty<string>(propertyBag,
                    BuildSourceFileNameBagNames.FormatKey(index), String.Empty);
                ctxProp.Namespace = ReadProperty<string>(propertyBag,
                    BuildSourceFileNameBagNames.FormatNamespace(index), String.Empty);
                ctxProp.IsLiteral = ReadProperty<bool>(propertyBag,
                    BuildSourceFileNameBagNames.FormatIsLiteral(index), true);

                NameParts.Add(ctxProp);
            }
        }

        public override void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
        {
            base.Save(propertyBag, clearDirty, saveAllProperties);

            int count = NameParts.Count;
            WriteProperty(propertyBag, BuildSourceFileNameBagNames.Count, count);

            for (int index = 0; index < count; index++)
            {
                LiteralOrProperty ctxProp = NameParts[index];

                WriteProperty(propertyBag,
                    BuildSourceFileNameBagNames.FormatKey(index), ctxProp.Key);
                WriteProperty(propertyBag,
                    BuildSourceFileNameBagNames.FormatNamespace(index), ctxProp.Namespace);
                WriteProperty(propertyBag,
                    BuildSourceFileNameBagNames.FormatIsLiteral(index), ctxProp.IsLiteral);
            }
        }

        #endregion
    }
}
