using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Resources;

using Microsoft.BizTalk.Component.Interop;

namespace Macaw.BizTalk.Pipeline
{
    public abstract class PipelineComponentWithResourceBase : PipelineComponentBase,
        IComponentUI
    {
        private CultureInfo _culture;
        private ResourceManager _rescMgr;

        protected PipelineComponentWithResourceBase(ResourceManager rescMgr)
            : this(String.Empty, rescMgr, CultureInfo.InvariantCulture)
        {}

        protected PipelineComponentWithResourceBase(string namePrefix, ResourceManager rescMgr)
            : this(namePrefix, rescMgr, CultureInfo.InvariantCulture)
        {}

        protected PipelineComponentWithResourceBase(string namePrefix, 
            ResourceManager rescMgr, CultureInfo culture)
        {
            if (rescMgr == null) throw new ArgumentNullException("rescMgr");

            _prefix = namePrefix;
            _rescMgr = rescMgr;
            _culture = culture;
        }

        private string _prefix;
        /// <summary>
        /// Gets or sets the prefix used in resource identifier names.
        /// </summary>
        protected string Prefix
        {
            get { return _prefix; }
            set { _prefix = value; }
        }

        /// <summary>
        /// Name of the component
        /// </summary>
        [Browsable(false)]
        public override string Name
        {
            get { return _rescMgr.GetString(_prefix + "ComponentName", _culture); }
        }

        /// <summary>
        /// Description of the component
        /// </summary>
        [Browsable(false)]
        public override string  Description
        {
            get { return _rescMgr.GetString(_prefix + "ComponentDescription", _culture); }
        }

        #region IComponentUI Members
        [Browsable(false)]
        public IntPtr Icon
        {
            get
            {
                object obj = _rescMgr.GetObject(_prefix + "ComponentIcon", _culture);

                if (obj == null)
                {
                    obj = _rescMgr.GetObject("ComponentIcon", CultureInfo.InvariantCulture);
                }

                Bitmap bmp = (Bitmap)obj;
                return bmp.GetHicon();
            }
        }

        public virtual IEnumerator Validate(object projectSystem)
        {
            return null;
        }
        #endregion
    }
}
